package com.ims.eval.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluationRevision;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluationRevisionService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 单位考评目标修订
 *
 * @author hlf
 * @date 2023/5/23 9:54
 * 文件说明：
 */
@Slf4j
@RestController
@RequestMapping("//evaluation-revision")
public class EvaluationRevisionController {

	@Autowired
	private IEvaluationRevisionService evaluationRevisionService;

	/**
	 * 单位考评目标修订列表信息（分页）
	 *
	 * @param pageNum            当前记录起始索引
	 * @param pageSize           每页显示记录数
	 * @param id                 主键
	 * @param responsibilityCode 业务编码
	 * @param cycleUnit          周期单位
	 * @param checkCycle         周期
	 * @param beginDate          考核开始时间
	 * @param endDate            考核截止时间
	 * @param stage              审批状态
	 * @param createBy           创建者
	 * @param year               年
	 * @param month              月
	 * @param des                描述
	 * @return 结果
	 */
	@GetMapping(value = "list")
	public R list(@RequestParam(value = "pageNum") Integer pageNum,
				  @RequestParam(value = "pageSize") Integer pageSize,
				  @RequestParam(value = "id", required = false) String id,
				  @RequestParam(value = "responsibilityCode", required = false) String responsibilityCode,
				  @RequestParam(value = "cycleUnit", required = false) String cycleUnit,
				  @RequestParam(value = "checkCycle", required = false) List<String> checkCycle,
				  @RequestParam(value = "beginDate", required = false) String beginDate,
				  @RequestParam(value = "endDate", required = false) String endDate,
				  @RequestParam(value = "stage", required = false) String stage,
				  @RequestParam(value = "createBy", required = false) String createBy,
				  @RequestParam(value = "year", required = false) String year,
				  @RequestParam(value = "month", required = false) String month,
				  @RequestParam(value = "des", required = false) String des) {
		IPage<EvaluationRevision> list = evaluationRevisionService.listPage(pageNum, pageSize, id, responsibilityCode, cycleUnit, checkCycle, beginDate, endDate, stage, createBy, year, month, des);
		return R.ok().data(list);
	}

	/**
	 * 单位考评目标修订信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "listAll")
	public R listAll() {
		List<EvaluationRevision> list = evaluationRevisionService.list();
		return R.ok().data(list);
	}

	/**
	 * 考评修订
	 *
	 * @param id 考评目标主键
	 * @return 结果
	 */
	@PostMapping(value = "/revise/{id}")
	public R revise(@PathVariable String id) {
		try {
			boolean b = evaluationRevisionService.save(id);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

}
