package com.ims.eval.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Dict;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.dao.EvaluationDeptBusinessPlanMapper;
import com.ims.eval.entity.*;
import com.ims.eval.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @author hlf
 * @date 2023/4/27 10:33
 * 文件说明：
 */
@Service
public class EvaluationDeptBusinessPlanServiceImpl extends ServiceImpl<EvaluationDeptBusinessPlanMapper, EvaluationDeptBusinessPlan> implements IEvaluationDeptBusinessPlanService {

	@Autowired
	private IEvaluationDeptBusinessContentService evaluationDeptBusinessContentService;

	@Autowired
	private IEvaluationDeptBusinessAssessmentService evaluationDeptBusinessAssessmentService;

	@Override
	public IPage<EvaluationDeptBusinessPlan> listPage(Integer pageNum, Integer pageSize, String responsibilityCode, String des, String appraisalYear, String stage) {
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}
		//构造分页构造器
		Page<EvaluationDeptBusinessPlan> page = new Page<>(pageNum, pageSize);
		return baseMapper.selectListPage(page, responsibilityCode, des, appraisalYear, stage);
	}

	@Override
	public boolean save(EvaluationDeptBusinessPlan evaluationDeptBusinessPlan) {
		boolean b;
		QueryWrapper<EvaluationDeptBusinessPlan> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(evaluationDeptBusinessPlan.getAppraisalYear())) {
			qw.lambda().eq(EvaluationDeptBusinessPlan::getAppraisalYear, evaluationDeptBusinessPlan.getAppraisalYear());
		}
		EvaluationDeptBusinessPlan businessPlan = baseMapper.selectOne(qw);
		if (ObjectUtil.isNotNull(businessPlan)) {
			return false;
		}
		String id = Convert.toStr(IdUtil.getSnowflake(1, 1).nextId());
		evaluationDeptBusinessPlan.setId(id);
		evaluationDeptBusinessPlan.setResponsibilityCode("BMKP_" + evaluationDeptBusinessPlan.getBusinessClass() + "_" + evaluationDeptBusinessPlan.getAppraisalYear());
		evaluationDeptBusinessPlan.setDocumentStatus("有效");
		evaluationDeptBusinessPlan.setStage("流程未启动");
		evaluationDeptBusinessPlan.setCreateTime(DateUtil.date());
		List<EvaluationDeptBusinessContent> evaluationDeptBusinessContentList = new ArrayList<>();
		JSONArray array = JSONUtil.parseArray(evaluationDeptBusinessPlan.getDeptData());
		List<Dict> deptList = JSONUtil.toList(array, Dict.class);
		StringBuilder deptIds = new StringBuilder();
		StringBuilder deptNames = new StringBuilder();
		StringBuilder deptCodes = new StringBuilder();
		for (Dict dict : deptList) {
			String deptId = dict.get("id").toString();
			String deptName = dict.get("keyName").toString();
			String deptCode = dict.get("dataKey").toString();
			EvaluationDeptBusinessContent evaluationDeptBusinessContent = new EvaluationDeptBusinessContent();
			evaluationDeptBusinessContent.setBusinessPlanId(id);
			evaluationDeptBusinessContent.setDeptId(deptId);
			evaluationDeptBusinessContent.setDeptName(deptName);
			evaluationDeptBusinessContent.setDeptCode(deptCode);
			evaluationDeptBusinessContent.setSerialNumber("1");
			evaluationDeptBusinessContent.setAuditStatus("-1");
			evaluationDeptBusinessContent.setCreateTime(DateUtil.date());
			evaluationDeptBusinessContentList.add(evaluationDeptBusinessContent);
			deptIds.append(deptId).append(",");
			deptNames.append(deptName).append(",");
			deptCodes.append(deptCode).append(",");
		}
		b = evaluationDeptBusinessContentService.saveBatch(evaluationDeptBusinessContentList);
		evaluationDeptBusinessPlan.setDeptId(deptIds.toString().substring(0, deptIds.toString().length() - 1));
		evaluationDeptBusinessPlan.setDeptName(deptNames.toString().substring(0, deptNames.toString().length() - 1));
		evaluationDeptBusinessPlan.setDeptCode(deptCodes.toString().substring(0, deptCodes.toString().length() - 1));
		b = super.save(evaluationDeptBusinessPlan);
		return b;
	}

	@Override
	public boolean editState(String id, String state, String instId) {
		EvaluationDeptBusinessPlan evaluationDeptBusinessPlan = baseMapper.selectById(id);
		if (StringUtils.isNotEmpty(state)) {
			if ("组织人事部领导审核".equals(state)) {
				state = "流程已结束";
				EvaluationDeptBusinessAssessment evaluationDeptBusinessAssessment = new EvaluationDeptBusinessAssessment();
				evaluationDeptBusinessAssessment.setId(evaluationDeptBusinessPlan.getId());
				evaluationDeptBusinessAssessment.setResponsibilityCode(evaluationDeptBusinessPlan.getResponsibilityCode());
				evaluationDeptBusinessAssessment.setDeptId(evaluationDeptBusinessPlan.getDeptId());
				evaluationDeptBusinessAssessment.setDeptName(evaluationDeptBusinessPlan.getDeptName());
				evaluationDeptBusinessAssessment.setDeptCode(evaluationDeptBusinessPlan.getDeptCode());
				evaluationDeptBusinessAssessment.setAppraisalYear(evaluationDeptBusinessPlan.getAppraisalYear());
				evaluationDeptBusinessAssessment.setEvaluationCategory(evaluationDeptBusinessPlan.getEvaluationCategory());
				evaluationDeptBusinessAssessment.setBusinessClass(evaluationDeptBusinessPlan.getBusinessClass());
				evaluationDeptBusinessAssessment.setDocumentStatus(evaluationDeptBusinessPlan.getDocumentStatus());
				evaluationDeptBusinessAssessment.setStage("流程未启动");
				evaluationDeptBusinessAssessment.setDes(evaluationDeptBusinessPlan.getDes());
				evaluationDeptBusinessAssessment.setReviewPeople(evaluationDeptBusinessPlan.getReviewPeople());
				evaluationDeptBusinessAssessment.setReviewPeopleTime(evaluationDeptBusinessPlan.getReviewPeopleTime());
				evaluationDeptBusinessAssessment.setReviewOpinion(evaluationDeptBusinessPlan.getReviewOpinion());
				evaluationDeptBusinessAssessment.setFinalReviewPeople(evaluationDeptBusinessPlan.getFinalReviewPeople());
				evaluationDeptBusinessAssessment.setFinalReviewPeopleTime(evaluationDeptBusinessPlan.getFinalReviewPeopleTime());
				evaluationDeptBusinessAssessment.setFinalReviewOpinion(evaluationDeptBusinessPlan.getFinalReviewOpinion());
				//evaluationDeptBusinessAssessment.setInstId(evaluationDeptBusinessPlan.getInstId());
				evaluationDeptBusinessAssessment.setCreateTime(DateUtil.date());
				evaluationDeptBusinessAssessment.setCreateBy(evaluationDeptBusinessPlan.getCreateBy());
				evaluationDeptBusinessAssessment.setUpdateTime(evaluationDeptBusinessPlan.getUpdateTime());
				evaluationDeptBusinessAssessment.setUpdateBy(evaluationDeptBusinessPlan.getUpdateBy());
				evaluationDeptBusinessAssessment.setRemark(evaluationDeptBusinessPlan.getRemark());
				evaluationDeptBusinessAssessmentService.save(evaluationDeptBusinessAssessment);
			}
			evaluationDeptBusinessPlan.setStage(state);
		}
		if (StringUtils.isNotEmpty(instId)) {
			evaluationDeptBusinessPlan.setInstId(instId);
		}
		return super.saveOrUpdate(evaluationDeptBusinessPlan);
	}

	@Override
	public boolean removeByIds(List<String> ids) {
		boolean b = false;
		for (String id : ids) {
			EvaluationDeptBusinessPlan evaluationDeptBusinessPlan = super.getById(id);
			if ("流程未启动".equals(evaluationDeptBusinessPlan.getStage()) || "流程已结束".equals(evaluationDeptBusinessPlan.getStage())) {
				b = super.removeById(id);
				b = evaluationDeptBusinessAssessmentService.removeById(id);
				b = evaluationDeptBusinessContentService.removeByBusinessPlanId(id);
			} else {
				return b;
			}
		}
		return b;
	}

	@Override
	public boolean updateById(EvaluationDeptBusinessPlan evaluationDeptBusinessPlan) {
		evaluationDeptBusinessPlan.setUpdateTime(DateUtil.date());
		EvaluationDeptBusinessAssessment evaluationDeptBusinessAssessment = new EvaluationDeptBusinessAssessment();
		evaluationDeptBusinessAssessment.setId(evaluationDeptBusinessPlan.getId());
		evaluationDeptBusinessAssessment.setDes(evaluationDeptBusinessPlan.getDes());
		evaluationDeptBusinessAssessmentService.updateById(evaluationDeptBusinessAssessment);
		return super.updateById(evaluationDeptBusinessPlan);
	}
}
